var $graphics = null;
var lastRenderT = 0;

function createPlanet(identifier, r, a, b, c, d, tx, ty)
{
    //console.log("createPlanet");
    return '<div class="planet" id="'+identifier+'" style="'+
            'width:'+r*2+'px;'+
            'height:'+r*2+'px;'+
            'border-radius:'+r+'px;'+
            '-moz-border-radius: '+r+'px;'+
            '-moz-transform: matrix('+a+','+b+','+c+','+d+','+tx+'px,'+ty+'px);'+
            '-webkit-transform: matrix('+a+','+b+','+c+','+d+','+tx+','+ty+');'+
            '-o-transform: matrix('+a+','+b+','+c+','+d+','+tx+','+ty+');'+
            'transform: matrix('+a+','+b+','+c+','+d+','+tx+','+ty+');'+
            '"></div>';
}

function setPlanet($identifier, data)
{
    //console.log("setPlanet");
    $identifier.css({
        'border-radius': data.r+'px',
        '-moz-border-radius': data.r+'px',
        'width':data.r*2,
        'height':data.r*2
    })
}


var GT = {
    rotate: function(deg) {
        
        var rad = parseFloat(deg) * (Math.PI/180.0),
            costheta = Math.cos(rad),
            sintheta = Math.sin(rad);

        var a = costheta,
            b = sintheta,
            c = -sintheta,
            d = costheta;

        return $M([
          [a, c, 0.0],
          [b, d, 0.0],
          [0.0, 0.0, 1.0]
        ]);
    },

    scale: function (sx, sy) {
        sx = sx || sx === 0 ? sx : 1;
        sy = sy || sy === 0 ? sy : 1;
        return $M([
          [sx, 0,  0],
          [0,  sy, 0],
          [0,  0,  1]
        ]);
    },

    scaleX: function (sx) {
        return this.scale(sx);
    },

    scaleY: function (sy) {
        return this.scale(1, sy);
    },
    skew: function (degX, degY) {
        var radX = parseFloat(degX) * (Math.PI/180),
            radY = parseFloat(degY) * (Math.PI/180),
            x = Math.tan(radX),
            y = Math.tan(radY);

        return $M([
          [1, x, 0],
          [y, 1, 0],
          [0, 0, 1]
        ]);
    },

    skewX: function (deg) {
        var rad = parseFloat(deg) * (Math.PI/180),
            x = Math.tan(rad);

        return $M([
          [1, x, 0],
          [0, 1, 0],
          [0, 0, 1]
        ]);

    },

    skewY: function (deg) {
        var rad = parseFloat(deg) * (Math.PI/180.0),
            y = Math.tan(rad);

        return $M([
          [1.0, 0.0, 0.0],
          [y, 1.0, 0.0],
          [0.0, 0.0, 1.0]
        ]);

    },
    translate: function (tx, ty) {
        tx = tx ? tx : 0.0;
        ty = ty ? ty : 0.0;

        return $M([
          [1.0, 0.0, tx],
          [0.0, 1.0, ty],
          [0.0, 0.0, 1.0]
        ]);
    },

    translateX: function (tx) {
        return this.translate(tx);
    },

    translateY: function (ty) {
        return this.translate(0, ty);
    },

    toCssMatrix: function(matrix) {
        return [matrix.e(1,1), matrix.e(2,1),
            matrix.e(1,2), matrix.e(2,2), matrix.e(1,3), matrix.e(2,3)
        ];

        /*return [Math.round(matrix.e(1,1)), Math.round(matrix.e(2,1)),
            Math.round(matrix.e(1,2)), Math.round(matrix.e(2,2)), Math.round(matrix.e(1,3)), Math.round(matrix.e(2,3))
        ];*/
    },
    toString: function(matrix) {
        return '['+matrix.e(1,1)+', '+matrix.e(2,1)+', '+matrix.e(1,2)+', '+matrix.e(2,2)+', '+matrix.e(1,3)+', '+matrix.e(2,3)+']';
    }
};

function setCssMatrix($identifier, matrix)
{
    var a = Math.round(matrix.e(1,1)),
        b = Math.round(matrix.e(2,1)),
        c = Math.round(matrix.e(1,2)),
        d = Math.round(matrix.e(2,2)),
        tx = Math.round(matrix.e(1,3)),
        ty = Math.round(matrix.e(2,3));

    $identifier.css({'-moz-transform':'matrix('+a+','+b+','+c+','+d+','+tx+'px,'+ty+'px)',
                    '-webkit-transform':'matrix('+a+','+b+','+c+','+d+','+tx+'px,'+ty+'px)',
                    '-o-transform':'matrix('+a+','+b+','+c+','+d+','+tx+'px,'+ty+'px)',
                    'transform':'matrix('+a+','+b+','+c+','+d+','+tx+'px,'+ty+'px)'});
}

//function movePlanet()

function render()
{
    var t = (new Date().getTime())/1000;
    var dt = t-lastRenderT;
    var origin_x = $graphics.width()/2.0;
    var origin_y = $graphics.height()/2.0;

    var TView = GT.translate(origin_x, origin_x);
    
    var TSun = GT.translate(sun.x0-sun.r, sun.y0-sun.r);

    var m1 =  TView.x(TSun);
    $('#timer').html(t+'; MSun='+GT.toString(m1)+'; TView='+GT.toString(TView)+'; TSun='+GT.toString(TSun));

    $sun.transform({ matrix:  GT.toCssMatrix(m1)}, { preserve: false });

    for(i=0;i<planets.length;i++) {
        planet = planets[i];
        //console.log(planet);
        m1 =  TView.x(planetCircularPositionMatrix(planet.data, t));
        planet.$id.transform({ matrix:  GT.toCssMatrix(m1)}, { preserve: false });
    }
    
}

function planetCircularPositionMatrix(planet, t)
{
    var alpha = (planet.w*t)%360.0;
    var TPos = GT.translate(planet.x0, planet.y0);
    var RRot = GT.rotate(alpha);
    var TCorection = GT.translate(-planet.r, -planet.r);

    return TCorection.x(RRot.x(TPos));
}